require 'spec_helper'

describe Sudoku::Cell do
  CELL_COORDINATES ||= Sudoku::CellCoordinates
  CELL ||= Sudoku::Cell
  let(:cell_coordinates_5_5) { CELL_COORDINATES.new(5, 5) }
  let(:cell_coordinates_7_7) { CELL_COORDINATES.new(7, 7) }

  context 'basic simple cells testing' do
    let(:test_cell) { CELL.new(cell_coordinates_5_5, false, 0) }
    let(:custom_cell) { CELL.new(cell_coordinates_5_5, true, 5) }

    it 'can create valid cell' do
      expect { test_cell }.not_to raise_error
      expect { custom_cell }.not_to raise_error
    end

    it 'has correct string representation' do
      expect(test_cell.to_s).to eql('[5, 5], 0, false')
    end

    it 'increments correctly' do
      (0..9).each do |i|
        expect(test_cell.value).to eql(i)
        test_cell.increment
      end
      expect(test_cell.value).to eql(0)
    end

    it 'decrements correctly' do
      test_cell.decrement
      (9.downto(0)).each do |i|
        expect(test_cell.value).to eql(i)
        test_cell.decrement
      end
      expect(test_cell.value).to eql(9)
    end

    it 'has correctly set parameters' do
      expect(test_cell.value).to eql(0)
      expect(test_cell.fixed).to eql(false)
      expect(test_cell.coordinates).to eql(cell_coordinates_5_5)

      expect(custom_cell.value).to eql(5)
      expect(custom_cell.fixed).to eql(true)
      expect(custom_cell.coordinates).to eql(cell_coordinates_5_5)

      expect(custom_cell.coordinates.coordinate_x).to eql(5)
      expect(custom_cell.coordinates.coordinate_y).to eql(5)

      expect(test_cell.coordinates.coordinate_x).to eql(5)
      expect(test_cell.coordinates.coordinate_y).to eql(5)
    end

    it 'the cells on same coordinates are equal' do
      expect(test_cell.eql?(custom_cell)).to be true
      expect(test_cell == custom_cell).to be true
    end

    it 'the cells with different coordinates are not equal' do
      expect(test_cell.eql?(CELL.new(cell_coordinates_7_7))).to be false
    end
  end

  it 'does not allow to create fixed 0 cell'  do
    expect { CELL.new(cell_coordinates_5_5, true, 0) }.to raise_error
  end

  it 'accepts only valid coordinates' do
    expect { CELL.new('whatever') }.to raise_error
  end

  it 'returns correct cell string representation' do
    expect(CELL.new(cell_coordinates_5_5, false, 0).as_string).to eql('')
    expect(CELL.new(cell_coordinates_5_5, true, 5).as_string).to eql('5')
  end

  context 'setting value to cell' do
    let(:fixed_cell) { CELL.new(cell_coordinates_7_7, true, 5) }
    let(:free_cell) { CELL.new(cell_coordinates_5_5, false, 0) }

    it 'cant assign value to fixed cell' do
      expect { fixed_cell.value = 5 }.to raise_error
    end

    it 'cant assign values out of interval 0-9' do
      expect { free_cell.value = -1 }.to raise_error
      expect { free_cell.value = 10 }.to raise_error
    end

    it 'can assign correct values' do
      expect { free_cell.value = 0 }.not_to raise_error
      expect { free_cell.value = 9 }.not_to raise_error
      expect { free_cell.value = 2 }.not_to raise_error
      expect(free_cell.value).to eql(2)
    end
  end
end
