require 'spec_helper'

describe Sudoku::Grid do
  CELL_COORDINATES ||= Sudoku::CellCoordinates
  CELL ||= Sudoku::Cell
  GRID ||= Sudoku::Grid

  let(:cell_coordinates_5_5) { CELL_COORDINATES.new(5, 5) }
  let(:cell_coordinates_7_7) { CELL_COORDINATES.new(7, 7) }
  let(:fixed_cell) { CELL.new(cell_coordinates_5_5, true, 5) }
  let(:free_cell) { CELL.new(cell_coordinates_7_7, false, 0) }
  let(:same_free_cell) { CELL.new(cell_coordinates_7_7, true, 7) }
  let(:test_grid) { GRID.new }
  let(:factory) { Sudoku::GridFactory.new }

  before { factory.disable_log }

  context 'adding elements' do
    it 'adds elements' do
      test_grid.add_cell(fixed_cell)
      cell_stored = test_grid.get_cell(cell_coordinates_5_5)
      expect(cell_stored).not_to be_nil
      expect(cell_stored.value).to eql(5)
      expect(cell_stored.fixed).to be true
      expect(cell_stored.coordinates == cell_coordinates_5_5).to be true
    end

    it 'replaces element which exists' do
      test_grid.add_cell(free_cell)
      test_grid.add_cell(same_free_cell)
      cell_stored = test_grid.get_cell(cell_coordinates_7_7)

      expect(cell_stored.value).to eql(same_free_cell.value)
      expect(cell_stored.coordinates).to eql(same_free_cell.coordinates)
      expect(cell_stored.fixed).to eql(same_free_cell.fixed)
    end
  end

  context 'getting elements' do
    let(:grid) { Sudoku::GridFactory.new.create_random }

    it 'returns default element, when element does not exist' do
      cell_stored = test_grid.get_cell(cell_coordinates_7_7)
      default_cell = test_grid.generate_default_cell(cell_coordinates_7_7)

      expect(cell_stored.value).to eql(default_cell.value)
      expect(cell_stored.coordinates).to eql(default_cell.coordinates)
      expect(cell_stored.fixed).to eql(default_cell.fixed)
    end
  end

  context 'misc operations' do
    let(:grid) { factory.create_random }
    let(:full_grid)  { factory.create_full_grid }
    let(:invalid_grid) do
      grid = factory.create_full_grid
      grid.add_cell(CELL.new(CELL_COORDINATES.new(0, 0), false, 5))
      grid.add_cell(CELL.new(CELL_COORDINATES.new(1, 0), false, 5))
      grid
    end

    it 'test filled properly' do
      expect(GRID.new.filled?).to be false
      expect(Sudoku::GridFactory.new.create_full_grid.filled?).to be true
    end

    it 'test grid violation' do
      expect(full_grid.rules_violated?).to be false
      expect(invalid_grid.rules_violated?).to be true
    end

    it 'test grid solved' do
      expect(full_grid.solved?).to be true
      expect(invalid_grid.solved?).to be false
      expect(grid.solved?).to be false
    end

    it 'resets cell' do
      default_cell = grid.generate_default_cell(cell_coordinates_5_5)
      grid.reset_cell(default_cell.coordinates)
      reset_cell = grid.get_cell(default_cell.coordinates)

      expect(default_cell.coordinates).to eql(reset_cell.coordinates)
      expect(default_cell.value).to eql(reset_cell.value)
      expect(default_cell.fixed).to eql(reset_cell.fixed)
      expect(default_cell == reset_cell).to be true
    end

    it 'has correct to_s method' do
      path = 'spec/sudoku/model/empty_grid.txt'
      empty_grid_s = GRID.new.to_s
      (File.open(path).zip empty_grid_s.lines).each do |x, y|
        expect(x).to eql(y)
      end
    end
  end
end
